@@ -1,5 +1,5 @@
 /* libmondo-devices.c                 Subroutines for handling devices
-   $Id: libmondo-devices.c 305 2006-01-12 16:37:42Z bcornec $
+   $Id: libmondo-devices.c 877 2006-10-05 21:47:22Z andree $
 .
 
 
@@ -251,7 +251,7 @@
 #endif
 
 /*@unused@*/
-//static char cvsid[] = "$Id: libmondo-devices.c 305 2006-01-12 16:37:42Z bcornec $";
+//static char cvsid[] = "$Id: libmondo-devices.c 877 2006-10-05 21:47:22Z andree $";
 
 extern int g_current_media_number;
 extern double g_kernel_version;
@@ -524,7 +524,7 @@ bool does_nonMS_partition_exist(void)
 #else
 	return
 		!system
-		("parted2fdisk -l 2>/dev/null | grep '^/dev/' | egrep -qv '(MS|DOS|FAT|NTFS)'");
+		("parted2fdisk -l 2>/dev/null | grep '^/dev/' | grep -Eqv '(MS|DOS|FAT|NTFS)'");
 #endif
 }
 
@@ -1878,10 +1878,12 @@ int interactively_obtain_media_parameters_from_user(struct s_bkpinfo
 	case cdrw:
 	case dvd:
 		if (archiving_to_media) {
-			if (ask_me_yes_or_no
-				("Is your computer a laptop, or does the CD writer incorporate BurnProof technology?"))
-			{
-				bkpinfo->manual_cd_tray = TRUE;
+			if (bkpinfo->backup_media_type != dvd) {
+				if (ask_me_yes_or_no
+					("Is your computer a laptop, or does the CD writer incorporate BurnProof technology?"))
+				{
+					bkpinfo->manual_cd_tray = TRUE;
+				}
 			}
 			if ((bkpinfo->compression_level =
 				 which_compression_level()) == -1) {
@@ -1959,7 +1961,7 @@ int interactively_obtain_media_parameters_from_user(struct s_bkpinfo
 			}
 			if (bkpinfo->media_device[0]) {
 				sprintf(tmp,
-						"I think I've found your %s burner at SCSI node %s; am I right on the money?",
+						"I think I've found your %s burner at SCSI node %s. Is this correct? (Say no if you have an IDE burner and you are running a 2.6 kernel. You will then be prompted for further details.)",
 						media_descriptor_string(bkpinfo->
 												backup_media_type),
 						bkpinfo->media_device);
@@ -2221,20 +2223,20 @@ int interactively_obtain_media_parameters_from_user(struct s_bkpinfo
 				for (i = 0; i <= MAX_NOOF_MEDIA; i++) {
 					bkpinfo->media_size[i] = atoi(sz_size);
 				}
-				if (!popup_and_get_string
-					("Prefix.",
-					 "Please enter the prefix that will be prepended to your ISO filename.  Example: machine1 to obtain machine1-[1-9]*.iso files",
-					 bkpinfo->prefix, MAX_STR_LEN / 4)) {
-					log_to_screen("User has chosen not to backup the PC");
-					finish(1);
-				}
-				log_msg(3, "prefix set to %s", bkpinfo->prefix);
 			} else {
 				for (i = 0; i <= MAX_NOOF_MEDIA; i++) {
 					bkpinfo->media_size[i] = 650;
 				}
 			}
 		}
+		if (!popup_and_get_string
+			("Prefix.",
+			 "Please enter the prefix that will be prepended to your ISO filename.  Example: machine1 to obtain machine1-[1-9]*.iso files",
+			 bkpinfo->prefix, MAX_STR_LEN / 4)) {
+			log_to_screen("User has chosen not to backup the PC");
+			finish(1);
+		}
+		log_msg(3, "prefix set to %s", bkpinfo->prefix);
 		break;
 	default:
 		fatal_error
@@ -2430,10 +2432,10 @@ char *list_of_NFS_devices_and_mounts(void)
 	malloc_string(exclude_these_directories);
 	strcpy(exclude_these_directories,
 		   call_program_and_get_last_line_of_output
-		   ("mount -t coda,ncpfs,nfs,smbfs,cifs | tr -s '\t' ' ' | cut -d' ' -f3 | tr -s '\n' ' ' | awk '{print $0;}'"));
+		   ("mount -t coda,ncpfs,nfs,smbfs,cifs,afs,mvfs | tr -s '\t' ' ' | cut -d' ' -f3 | tr -s '\n' ' ' | awk '{print $0;}'"));
 	strcpy(exclude_these_devices,
 		   call_program_and_get_last_line_of_output
-		   ("tr -s '\t' ' ' < /etc/fstab | grep -E '( (coda|ncpfs|nfs|smbfs|cifs) )' | cut -d' ' -f1 | tr -s '\n' ' ' | awk '{print $0;}'"));
+		   ("tr -s '\t' ' ' < /etc/fstab | grep -E '( (coda|ncpfs|nfs|smbfs|cifs|mvfs) )' | cut -d' ' -f1 | tr -s '\n' ' ' | awk '{print $0;}'"));
 	sprintf(result_sz, "%s %s", exclude_these_directories,
 			exclude_these_devices);
 	paranoid_free(exclude_these_devices);
@@ -2460,10 +2462,10 @@ char *list_of_NFS_mounts_only(void)
 	malloc_string(exclude_these_directories);
 	strcpy(exclude_these_directories,
 		   call_program_and_get_last_line_of_output
-		   ("mount -t coda,ncpfs,nfs,smbfs,cifs | tr -s '\t' ' ' | cut -d' ' -f3 | tr -s '\n' ' ' | awk '{print $0;}'"));
+		   ("mount -t coda,ncpfs,nfs,smbfs,cifs,afs,mvfs | tr -s '\t' ' ' | cut -d' ' -f3 | tr -s '\n' ' ' | awk '{print $0;}'"));
 	strcpy(exclude_these_devices,
 		   call_program_and_get_last_line_of_output
-		   ("tr -s '\t' ' ' < /etc/fstab | grep -E '( (coda|ncpfs|nfs|smbfs|cifs) )' | cut -d' ' -f1 | tr -s '\n' ' ' | awk '{print $0;}'"));
+		   ("tr -s '\t' ' ' < /etc/fstab | grep -E '( (coda|ncpfs|nfs|smbfs|cifs|mvfs) )' | cut -d' ' -f1 | tr -s '\n' ' ' | awk '{print $0;}'"));
 	sprintf(result_sz, "%s", exclude_these_directories);
 	paranoid_free(exclude_these_devices);
 	paranoid_free(exclude_these_directories);
@@ -2522,11 +2524,11 @@ void sensibly_set_tmpdir_and_scratchdir(struct s_bkpinfo *bkpinfo)
 #ifdef __FreeBSD__
 	strcpy(tmp,
 		   call_program_and_get_last_line_of_output
-		   ("df -m -P -t nonfs,msdosfs,ntfs,smbfs,smb,cifs | tr -s '\t' ' ' | grep -v none | grep -v Filesystem | awk '{printf \"%s %s\\n\", $4, $6;}' | sort -n | tail -n1 | awk '{print $NF;}'"));
+		   ("df -m -P -t nonfs,msdosfs,ntfs,smbfs,smb,cifs,afs,mvfs | tr -s '\t' ' ' | grep -vE \"none|Filesystem\" | awk '{printf \"%s %s\\n\", $4, $6;}' | sort -n | tail -n1 | awk '{print $NF;}'"));
 #else
 	strcpy(tmp,
 		   call_program_and_get_last_line_of_output
-		   ("df -m -P -x nfs -x vfat -x ntfs -x smbfs -x smb -x cifs | sed 's/                  /devdev/' | tr -s '\t' ' ' | grep -v none | grep -v Filesystem | grep -v /dev/shm | awk '{printf \"%s %s\\n\", $4, $6;}' | sort -n | tail -n1 | awk '{print $NF;}'"));
+		   ("df -m -P -x nfs -x vfat -x ntfs -x smbfs -x smb -x cifs -x afs -x mvfs | sed 's/                  /devdev/' | tr -s '\t' ' ' | grep -vE \"none|Filesystem|/dev/shm\" | awk '{printf \"%s %s\\n\", $4, $6;}' | sort -n | tail -n1 | awk '{print $NF;}'"));
 #endif
 
 	if (tmp[0] != '/') {
