@@ -1,5 +1,5 @@
 /* libmondo-filelist.c
-   $Id: libmondo-filelist.c 336 2006-01-19 22:32:46Z bcornec $
+   $Id: libmondo-filelist.c 847 2006-09-26 22:28:48Z bruno $
 
 - for subroutines which manipulate the filelist
 
@@ -116,7 +116,7 @@
 #include "libmondo-fork-EXT.h"
 #include "libmondo-gui-EXT.h"
 #include "libmondo-tools-EXT.h"
-
+#include "mr_string.h"
 
 #include <time.h>
 #include <stdio.h>
@@ -143,7 +143,7 @@ int mondo_makefilelist(char *logfile, char *tmpdir, char *scratchdir,
 
 
 /*@unused@*/
-//static char cvsid[] = "$Id: libmondo-filelist.c 336 2006-01-19 22:32:46Z bcornec $";
+//static char cvsid[] = "$Id: libmondo-filelist.c 847 2006-09-26 22:28:48Z bruno $";
 
 /**
  * Number of lines in the filelist last loaded.
@@ -591,7 +591,7 @@ int gen_aux_list(char *filelist, char *syscall_sprintf,
 			file_to_analyze[i - 1] = '\0';
 		}
 		log_msg(8, "Analyzing %s", file_to_analyze);
-		sprintf(syscall, syscall_sprintf, file_to_analyze);
+		sprintf(syscall, syscall_sprintf, mr_stresc(file_to_analyze, "`$\\\"", '\\'));
 		strcat(syscall, " 2>> /dev/null");	// " MONDO_LOGFILE);
 		call_exe_and_pipe_output_to_fd(syscall, pout);
 	}
@@ -1437,14 +1437,6 @@ int prepare_filelist(struct s_bkpinfo *bkpinfo)
 
 	/*@ int **************************************************** */
 	int res = 0;
-// patch by Herman Kuster
-// end patch
-	int *p_res = &res;
-
-	/*@ buffers ************************************************ */
-//  char command[MAX_STR_LEN*2];
-
-	/*@ i don't have any idea ********************************** */
 
 	assert(bkpinfo != NULL);
 	log_it("tmpdir=%s; scratchdir=%s", bkpinfo->tmpdir,
@@ -1473,8 +1465,7 @@ int prepare_filelist(struct s_bkpinfo *bkpinfo)
 	}
 
 	if (res) {
-		log_OS_error("Call to mondo-makefilelist failed");
-		*p_res++;
+		log_OS_error("Call to mondo_makefilelist failed");
 		mvaddstr_and_log_it(g_currentY++, 74, "Failed.");
 	} else {
 		mvaddstr_and_log_it(g_currentY++, 74, "Done.");
@@ -1495,6 +1486,8 @@ int prepare_filelist(struct s_bkpinfo *bkpinfo)
 int open_and_list_dir(char *dir, char *sth, FILE * fout,
 					  time_t time_of_last_full_backup)
 {
+	const char delims[] = " ";
+
 	DIR *dip;
 	struct dirent *dit;
 	struct stat statbuf;
@@ -1505,8 +1498,12 @@ int open_and_list_dir(char *dir, char *sth, FILE * fout,
 	char *ith_B;
 	char *skip_these;
 	char *new_with_spaces;
+	char *strtmp;
+	char *token;
+	char *find_excludes;
 	static char *name_of_evalcall_form;
 	int i;
+	int lastpos = 0;
 	static int depth = 0;
 	char *p;
 	static int counter = 0;
@@ -1530,18 +1527,29 @@ int open_and_list_dir(char *dir, char *sth, FILE * fout,
 	if (!depth) {
 		malloc_string(name_of_evalcall_form);
 		malloc_string(find_skeleton_marker);
+		asprintf(&find_excludes, " ");
+		while((token = mr_strtok (sth, delims, &lastpos))) {
+		  asprintf(&strtmp,"%s", find_excludes);
+		  paranoid_free(find_excludes);
+		  asprintf(&find_excludes,"%s -path %s -prune -o", strtmp, token);
+		  paranoid_free(strtmp);
+		  paranoid_free(token);
+		}
 #if linux
 		// 2.6 has /sys as a proc-type thing -- must be excluded
-		sprintf(tmp,
-				"find %s -maxdepth %d -path /proc -prune -o -path /sys -prune -o -path /dev/shm -prune -o -path /media/floppy -prune -o -type d -a -print > %s 2> /dev/null",
-				dir, MAX_SKEL_DEPTH, g_skeleton_filelist);
+		asprintf(&strtmp,
+			 "find %s -maxdepth %d -fstype mvfs -prune -o -path /dev/shm -prune -o %s -type d -print > %s 2> /dev/null",
+			 dir, MAX_SKEL_DEPTH, find_excludes, g_skeleton_filelist);
 #else
 		// On BSD, for example, /sys is the kernel sources -- don't exclude
-		sprintf(tmp,
-				"find %s -maxdepth %d -path /proc -prune -o -type d -a -print > %s 2> /dev/null",
-				dir, MAX_SKEL_DEPTH, g_skeleton_filelist);
+		asprintf(&strtmp,
+			 "find %s -maxdepth %d -fstype mvfs -prune -o -path /proc -prune -o %s -type d -print > %s 2> /dev/null",
+				dir, MAX_SKEL_DEPTH, find_excludes, g_skeleton_filelist);
 #endif
-		system(tmp);
+		paranoid_free(find_excludes);
+		log_msg(5, "find command = %s", strtmp);
+		system(strtmp);
+		paranoid_free(strtmp);
 		sprintf(tmp, "wc -l %s | awk '{print $1;}'", g_skeleton_filelist);
 		g_skeleton_entries =
 			1 + atol(call_program_and_get_last_line_of_output(tmp));
@@ -1554,7 +1562,7 @@ int open_and_list_dir(char *dir, char *sth, FILE * fout,
 	} else if (depth <= MAX_SKEL_DEPTH)	// update evalcall form if appropriate
 	{
 		sprintf(find_skeleton_marker,
-				"fgrep -v \"%s\" %s > %s.new 2> /dev/null", dir,
+				"grep -Fv \"%s\" %s > %s.new 2> /dev/null", dir,
 				g_skeleton_filelist, g_skeleton_filelist);
 //    log_msg(0, "fsm = %s", find_skeleton_marker);
 		if (!system(find_skeleton_marker)) {
