@@ -1,5 +1,5 @@
 /* libmondo-files.c                                  file manipulation
-   $Id: libmondo-files.c 234 2005-12-17 23:36:51Z bcornec $
+   $Id: libmondo-files.c 879 2006-10-07 21:16:48Z bruno $
 .
 
 
@@ -96,7 +96,7 @@
 #include "libmondo-string-EXT.h"
 
 /*@unused@*/
-//static char cvsid[] = "$Id: libmondo-files.c 234 2005-12-17 23:36:51Z bcornec $";
+//static char cvsid[] = "$Id: libmondo-files.c 879 2006-10-07 21:16:48Z bruno $";
 
 extern char err_log_lines[NOOF_ERR_LINES][MAX_STR_LEN];
 
@@ -340,12 +340,24 @@ void exclude_nonexistent_files(char *inout)
 int figure_out_kernel_path_interactively_if_necessary(char *kernel)
 {
 	char tmp[MAX_STR_LEN];
+	char *command;
 
 	if (!kernel[0]) {
 		strcpy(kernel,
 			   call_program_and_get_last_line_of_output
 			   ("mindi --findkernel 2> /dev/null"));
 	}
+	// If we didn't get anything back, check whether mindi raised a fatal error
+	if (!kernel[0]) {
+		malloc_string(command);
+		strcpy(command, "grep 'Fatal error' /var/log/mindi.log");
+		strcpy(tmp, call_program_and_get_last_line_of_output(command));
+		if (strlen(tmp) > 1) {
+			popup_and_OK(tmp);
+			fatal_error("Mindi gave a fatal error. Please check '/var/log/mindi.log'.");
+		}
+		paranoid_free(command);
+	}
 	log_it("Calling Mindi with kernel path of '%s'", kernel);
 	while (!kernel[0]) {
 		if (!ask_me_yes_or_no
@@ -507,7 +519,7 @@ int grab_percentage_from_last_line_of_file(char *filename)
 		 && !strstr(err_log_lines[i], "% done"); i--);
 	if (i < 0) {
 		sprintf(command,
-				"tail -n3 %s | fgrep -i \"%c\" | tail -n1 | awk '{print $0;}'",
+				"tail -n3 %s | grep -Fi \"%c\" | tail -n1 | awk '{print $0;}'",
 				filename, '%');
 		strcpy(lastline,
 			   call_program_and_get_last_line_of_output(command));
@@ -583,13 +595,13 @@ char *last_line_of_file(char *filename)
  * @param filename The file to get the length of.
  * @return The length of the file, or -1 for error.
  */
-long long length_of_file(char *filename)
+off_t length_of_file(char *filename)
 {
 	/*@ pointers *************************************************** */
 	FILE *fin;
 
 	/*@ long long ************************************************* */
-	long long length;
+	off_t length;
 
 	fin = fopen(filename, "r");
 	if (!fin) {
@@ -597,8 +609,8 @@ long long length_of_file(char *filename)
 		log_OS_error("Unable to openin filename");
 		return (-1);
 	}
-	fseek(fin, 0, SEEK_END);
-	length = ftell(fin);
+	fseeko(fin, 0, SEEK_END);
+	length = ftello(fin);
 	paranoid_fclose(fin);
 	return (length);
 }
@@ -628,8 +640,8 @@ make_checksum_list_file(char *filelist, char *cksumlist, char *comppath)
 	char tmp[1000];
 
 	/*@ long [long] ************************************************* */
-	long long filelist_length;
-	long curr_pos;
+	off_t filelist_length;
+	off_t curr_pos;
 	long start_time;
 	long current_time;
 	long time_taken;
@@ -667,7 +679,7 @@ make_checksum_list_file(char *filelist, char *cksumlist, char *comppath)
 			current_time = get_time();
 			counter = 0;
 			curr_fname[37] = '\0';
-			curr_pos = ftell(fin) / 1024;
+			curr_pos = ftello(fin) / 1024;
 			percentage = (int) (curr_pos * 100 / filelist_length);
 			time_taken = current_time - start_time;
 			if (percentage == 0) {
@@ -844,8 +856,8 @@ long size_of_partition_in_mountlist_K(char *tmpdir, char *dev)
 
 	sprintf(mountlist, "%s/mountlist.txt", tmpdir);
 	sprintf(command,
-			"cat %s/mountlist.txt | grep \"%s \" | head -n1 | awk '{print $4;}'",
-			tmpdir, dev);
+			"grep \"%s \" %s/mountlist.txt | head -n1 | awk '{print $4}'",
+			dev, tmpdir);
 	log_it(command);
 	strcpy(sz_res, call_program_and_get_last_line_of_output(command));
 	file_len_K = atol(sz_res);
@@ -864,6 +876,8 @@ long size_of_all_biggiefiles_K(struct s_bkpinfo *bkpinfo)
 	char *fname;
 	char *biggielist;
 	char *comment;
+	char *tmp;
+	char *command;
 
 	/*@ long ******************************************************** */
 	long scratchL = 0;
@@ -877,6 +891,8 @@ long size_of_all_biggiefiles_K(struct s_bkpinfo *bkpinfo)
 	malloc_string(fname);
 	malloc_string(biggielist);
 	malloc_string(comment);
+	malloc_string(tmp);
+	malloc_string(command);
 	log_it("Calculating size of all biggiefiles (in total)");
 	sprintf(biggielist, "%s/biggielist.txt", bkpinfo->tmpdir);
 	log_it("biggielist = %s", biggielist);
@@ -891,8 +907,20 @@ long size_of_all_biggiefiles_K(struct s_bkpinfo *bkpinfo)
 				fname[strlen(fname) - 1] = '\0';
 			}
 			if (0 == strncmp(fname, "/dev/", 5)) {
-				file_len_K = get_phys_size_of_drive(fname) * 1024L;
+				if (is_dev_an_NTFS_dev(fname)) {
+					if ( !find_home_of_exe("ntfsresize")) {
+						fatal_error("ntfsresize not found");
+					}
+					sprintf(command, "ntfsresize --force --info %s|grep '^You might resize at '|cut -d' ' -f5", fname);
+					log_it("command = %s", command);
+					strcpy (tmp, call_program_and_get_last_line_of_output(command));
+					log_it("res of it = %s", tmp);
+					file_len_K = atoll(tmp) / 1024L;
+				} else {
+					file_len_K = get_phys_size_of_drive(fname) * 1024L;
+				}
 			} else {
+				/* BERLIOS: more than long here ??? */
 				file_len_K = (long) (length_of_file(fname) / 1024);
 			}
 			if (file_len_K > 0) {
@@ -914,6 +942,8 @@ long size_of_all_biggiefiles_K(struct s_bkpinfo *bkpinfo)
 	paranoid_free(fname);
 	paranoid_free(biggielist);
 	paranoid_free(comment);
+	paranoid_free(tmp);
+	paranoid_free(command);
 	return (scratchL);
 }
 
@@ -936,17 +966,24 @@ long long space_occupied_by_cd(char *mountpt)
 	/*@ end vars *************************************************** */
 
 	sprintf(command, "du -sk %s", mountpt);
+	errno = 0;
 	fin = popen(command, "r");
-	(void) fgets(tmp, MAX_STR_LEN, fin);
-	paranoid_pclose(fin);
-	p = strchr(tmp, '\t');
-	if (p) {
-		*p = '\0';
-	}
-	for (p = tmp, llres = 0; *p != '\0'; p++) {
-		llres *= 10;
-		llres += (int) (*p - '0');
+	if (errno) {
+	  log_it("popen() FAILED: command=%s, mountpt=%s, fin=%d, errno=%d, strerror=%s", command, mountpt, fin, errno, strerror(errno));
+	  llres = 0;
+	} else {
+	  (void) fgets(tmp, MAX_STR_LEN, fin);
+	  paranoid_pclose(fin);
+	  p = strchr(tmp, '\t');
+	  if (p) {
+	    *p = '\0';
+	  }
+	  for (p = tmp, llres = 0; *p != '\0'; p++) {
+	    llres *= 10;
+	    llres += (int) (*p - '0');
+	  }
 	}
+
 	return (llres);
 }
 
@@ -1183,8 +1220,8 @@ void store_nfs_config(struct s_bkpinfo *bkpinfo)
 {
 
 	/*@ buffers ******** */
-	char outfile[MAX_STR_LEN];
 	char nfs_dev[MAX_STR_LEN];
+	char mac_addr[MAX_STR_LEN];
 	char nfs_mount[MAX_STR_LEN];
 	char nfs_client_ipaddr[MAX_STR_LEN];
 	char nfs_client_netmask[MAX_STR_LEN];
@@ -1196,9 +1233,6 @@ void store_nfs_config(struct s_bkpinfo *bkpinfo)
 
 	/*@ pointers ***** */
 	char *p;
-	FILE *fout;
-
-
 
 	log_it("Storing NFS configuration");
 	strcpy(tmp, bkpinfo->nfs_mount);
@@ -1210,6 +1244,7 @@ void store_nfs_config(struct s_bkpinfo *bkpinfo)
 	*(p++) = '\0';
 	strcpy(nfs_server_ipaddr, tmp);
 	strcpy(nfs_mount, p);
+	/* BERLIOS : there is a bug #67 here as it only considers the first NIC */
 	sprintf(command,
 			"ifconfig | tr '\n' '#' | sed s/##// | tr '#' ' ' | tr '' '\n' | head -n1 | cut -d' ' -f1");
 	strcpy(nfs_dev, call_program_and_get_last_line_of_output(command));
@@ -1236,50 +1271,26 @@ void store_nfs_config(struct s_bkpinfo *bkpinfo)
 		fatal_error
 			("Unable to find ethN (eth0, eth1, ...) adapter via NFS mount you specified.");
 	}
-	sprintf(outfile, "%s/start-nfs", bkpinfo->tmpdir);
-	sprintf(tmp, "outfile = %s", outfile);
-	log_it(tmp);
-	if (!(fout = fopen(outfile, "w"))) {
-		fatal_error("Cannot store NFS config");
-	}
-	fprintf(fout, "#!/bin/sh\n");
-	fprintf(fout, "# number of ping\n");
-	fprintf(fout, "ipcount=3\n");
-    fprintf(fout, "for i in `cat /proc/cmdline` ; do\n");
-    fprintf(fout, "    echo $i | grep -qi ping= && ipcount=`echo $i | cut -d= -f2`\n");
-    fprintf(fout, "done\n");
-	fprintf(fout, "ifconfig lo 127.0.0.1  # config loopback\n");
-	fprintf(fout, "ipaddress=%s\n", nfs_client_ipaddr);
-	fprintf(fout, "ipnetmask=%s\n", nfs_client_netmask);
-	fprintf(fout, "ipbroadcast=%s\n", nfs_client_broadcast);
-	fprintf(fout, "ipgateway=%s\n", nfs_client_defgw);
-	fprintf(fout, "ipconf=\n");
-    fprintf(fout, "for i in `cat /proc/cmdline` ; do\n");
-    fprintf(fout, "    echo $i | grep -qi ipconf= && ipconf=`echo $i | cut -d= -f2`\n");
-    fprintf(fout, "done\n");
-    fprintf(fout, "if [ \"$ipconf\" = \"dhcp\" ]; then\n");
-    fprintf(fout, "    udhcpc -i %s\n", nfs_dev);
-    fprintf(fout, "else\n");
-    fprintf(fout, "    if [ \"$ipconf\" != \"\" ]; then\n");
-    fprintf(fout, "        ipaddress=`echo $ipconf | cut -d: -f1`\n");
-    fprintf(fout, "        ipnetmask=`echo $ipconf | cut -d: -f2`\n");
-    fprintf(fout, "        ipbroadcast=`echo $ipconf | cut -d: -f3`\n");
-    fprintf(fout, "        ipgateway=`echo $ipconf | cut -d: -f4`\n");
-    fprintf(fout, "    fi\n");
-	fprintf(fout, "    ifconfig %s $ipaddress netmask $ipnetmask broadcast $ipbroadcast\n", nfs_dev);
-	fprintf(fout, "    route add default gw $ipgateway\n");
-    fprintf(fout, "fi\n");
-	fprintf(fout, "ping -c $ipcount %s	# ping server\n", nfs_server_ipaddr);
-	fprintf(fout, "mount -t nfs -o nolock %s /tmp/isodir\n",
-			bkpinfo->nfs_mount);
-	paranoid_fclose(fout);
-	chmod(outfile, 0777);
 	make_hole_for_dir("/var/cache/mondo-archive");
-
-//  paranoid_system ("mkdir -p /var/cache/mondo-archive 2> /dev/null");
-
-	sprintf(tmp, "cp -f %s /var/cache/mondo-archive", outfile);
-	run_program_and_log_output(tmp, FALSE);
+	/********
+	* If the NFS device that found above is a bonded device, 
+	* we need to replace it with an ethN device or the
+	* networking will not start during an NFS restore.
+	*
+	* If the NFS device in nfs_dev begins with the word "bond", 
+	* look for the corresponding slave ethN device and copy it to nfs_dev.
+	* Using the common MAC address
+	********/
+	if (!strncmp(nfs_dev, "bond", 4)) {
+		log_to_screen("Found bonding device %s; looking for corresponding ethN slave device\n", nfs_dev);
+		sprintf(command,
+				"ifconfig %s | awk '{print $5}'", nfs_dev);
+		strcpy(mac_addr, call_program_and_get_last_line_of_output(command));
+		sprintf(command,
+				"ifconfig | grep -E '%s' | grep -v '%s' | head -n1 | cut -d' ' -f1", mac_addr,nfs_dev );
+		strcpy(nfs_dev, call_program_and_get_last_line_of_output(command));
+		log_to_screen("Replacing it with %s\n", nfs_dev);
+	}
 
 	sprintf(tmp, "%s/NFS-DEV", bkpinfo->tmpdir);
 	write_one_liner_data_file(tmp, nfs_dev);
