@@ -1,5 +1,5 @@
 /* libmondo-fork.c
-   $Id: libmondo-fork.c 411 2006-02-20 10:14:40Z andree $
+   $Id: libmondo-fork.c 797 2006-09-16 03:36:20Z andree $
 
 - subroutines for handling forking/pthreads/etc.
 
@@ -111,7 +111,7 @@
 #include "lib-common-externs.h"
 
 /*@unused@*/
-//static char cvsid[] = "$Id: libmondo-fork.c 411 2006-02-20 10:14:40Z andree $";
+//static char cvsid[] = "$Id: libmondo-fork.c 797 2006-09-16 03:36:20Z andree $";
 
 extern char *g_tmpfs_mountpt;
 extern t_bkptype g_backup_media_type;
@@ -615,7 +615,7 @@ int copy_from_src_to_dest(FILE * f_orig, FILE * f_archived, char direction)
 				log_msg(1, "bytes_read_in = %ld", bytes_read_in);
 //      if (bytes_read_in!=128+64) { fatal_error("Can't read the terminating block"); }
 				fwrite(tmp, 1, bytes_read_in, ftmp);
-				sprintf(tmp, "I am here - %ld", ftell(fin));
+				sprintf(tmp, "I am here - %lld", (long long)ftello(fin));
 //    log_msg(0, tmp);
 				fread(tmp, 1, 512, fin);
 				log_msg(0, "tmp = '%s'", tmp);
