@@ -1,5 +1,5 @@
 /*
- * $Id: libmondo-string-EXT.h 576 2006-05-25 12:00:37Z bcornec $
+ * $Id: libmondo-string-EXT.h 831 2006-09-25 11:07:16Z andree $
  */
 
 extern char *build_partition_name(char *partition, const char *drive,
@@ -43,5 +43,3 @@ extern char *media_descriptor_string(t_bkptype);
 
 extern inline void turn_wildcard_chars_into_literal_chars(char *out,
 														  char *in);
-/* Valid external functions */
-extern char *mr_strtok(char *instr, const char *delims, int *lastpos);
