@@ -1,4 +1,4 @@
-/* $Id: libmondo-string.c 576 2006-05-25 12:00:37Z bcornec $ */
+/* $Id: libmondo-string.c 841 2006-09-26 12:47:02Z andree $ */
 
 
 /**
@@ -15,7 +15,7 @@
 #include "libmondo-tools-EXT.h"
 
 /*@unused@*/
-//static char cvsid[] = "$Id: libmondo-string.c 576 2006-05-25 12:00:37Z bcornec $";
+//static char cvsid[] = "$Id: libmondo-string.c 841 2006-09-26 12:47:02Z andree $";
 
 extern int g_current_media_number;
 extern long long g_tape_posK;
@@ -97,6 +97,7 @@ void center_string(char *in_out, int width)
 	}
 	for (p = in_out; *p == ' '; p++);
 	strcpy(scratch, p);
+	strip_spaces (scratch);
 	len = (int) strlen(scratch);
 	mid = width / 2;
 	x = mid - len / 2;
@@ -105,6 +106,10 @@ void center_string(char *in_out, int width)
 	}
 	in_out[i] = '\0';
 	strcat(in_out, scratch);
+	for (i = x + len ; i < width - 1; i++) {
+      		in_out[i] = ' ';
+    	}
+  	in_out[i] = '\0';
 }
 
 
@@ -1155,31 +1160,3 @@ char *media_descriptor_string(t_bkptype type_of_bkp)
 	}
 	return (type_of_backup);
 }
-
-
-/* New functions safe from a memory manageemnt point of view */
-/* Developped by Andree Leidenfrost */
-
-char *mr_strtok(char *instr, const char *delims, int *lastpos) {
-
-char *token = NULL;
-char *strptr = NULL;
-size_t pos1 = 0;
-size_t pos2 = 0;
-
-if (strlen(instr) <= *lastpos) {
-	*lastpos = 0;
-	return token;
-}
-
-strptr = instr + *lastpos;
-pos2 = strspn(strptr, delims);
-strptr += pos2;
-pos1 = strcspn(strptr, delims);
-token = malloc(sizeof(*token)*(pos1+1));
-strncpy(token, strptr, pos1);
-*lastpos = *lastpos + pos1 + pos2 + 1;
-
-return token;
-}
-/* @} - end of stringGroup */
