@@ -1,5 +1,5 @@
 /* libmondo-tools.c                                  misc tools
-   $Id: libmondo-tools.c 576 2006-05-25 12:00:37Z bcornec $
+   $Id: libmondo-tools.c 686 2006-07-06 11:19:31Z andree $
 .
 
 
@@ -185,7 +185,7 @@
 #include <arpa/inet.h>
 
 /*@unused@*/
-//static char cvsid[] = "$Id: libmondo-tools.c 576 2006-05-25 12:00:37Z bcornec $";
+//static char cvsid[] = "$Id: libmondo-tools.c 686 2006-07-06 11:19:31Z andree $";
 
 extern int g_tape_buffer_size_MB;
 extern char *g_erase_tmpdir_and_scratchdir;
@@ -585,7 +585,7 @@ int post_param_configuration(struct s_bkpinfo *bkpinfo)
 	malloc_string(iso_tmp);
 	malloc_string(iso_path);
 	bkpinfo->optimal_set_size =
-		(IS_THIS_A_STREAMING_BACKUP(bkpinfo->backup_media_type) ? 4 : 8) *
+		(IS_THIS_A_STREAMING_BACKUP(bkpinfo->backup_media_type) ? 16 : 16) *
 		1024;
 
 	log_msg(1, "Foo");
@@ -604,6 +604,7 @@ int post_param_configuration(struct s_bkpinfo *bkpinfo)
 		make_hole_for_dir(bkpinfo->isodir);
 
 	run_program_and_log_output("uname -a", 5);
+	run_program_and_log_output("cat /etc/*-release", 5);
 	run_program_and_log_output("cat /etc/*issue*", 5);
 	sprintf(g_tmpfs_mountpt, "%s/tmpfs", bkpinfo->tmpdir);
 	sprintf(command, "mkdir -p %s", g_tmpfs_mountpt);
@@ -696,7 +697,11 @@ int post_param_configuration(struct s_bkpinfo *bkpinfo)
 					extra_cdrom_params, bkpinfo->media_device);
 		}
 		if (getenv ("SUDO_COMMAND")) {
-			fatal_error("Can't write DVDs as sudo because growisofs doesn't support this - see the growisofs manpage for details.");
+			sprintf(command, "strings `which growisofs` | grep -c SUDO_COMMAND");
+			if (!strcmp(call_program_and_get_last_line_of_output(command), "1")) {
+				popup_and_OK("Fatal Error: Can't write DVDs as sudo because growisofs doesn't support this - see the growisofs manpage for details.");
+				fatal_error("Can't write DVDs as sudo because growisofs doesn't support this - see the growisofs manpage for details.");
+			}		
 		}
 		log_msg(2, "call_make_iso (DVD res) is ... %s",
 				bkpinfo->call_make_iso);
@@ -1124,10 +1129,10 @@ int some_basic_system_sanity_checks()
 
 	// abort if Windows partition but no ms-sys and parted
 	if (!run_program_and_log_output
-		("mount | grep -w vfat | grep -v /dev/fd | grep -v nexdisk", 0)
+		("mount | grep -w vfat | grep -vE \"/dev/fd|nexdisk\"", 0)
 		||
 		!run_program_and_log_output
-		("mount | grep -w dos | grep -v /dev/fd | grep -v nexdisk", 0)) {
+		("mount | grep -w dos | grep -vE \"/dev/fd|nexdisk\"", 0)) {
 		log_to_screen("I think you have a Windows 9x partition.");
 		retval += whine_if_not_found("parted");
 #ifndef __IA64__
