@@ -1,5 +1,5 @@
 /* newt-specific.c
-   $Id: newt-specific.c 586 2006-05-29 09:36:33Z bcornec $
+   $Id: newt-specific.c 877 2006-10-05 21:47:22Z andree $
 
   subroutines which do display-type things
   and use the newt library to do them
@@ -121,12 +121,13 @@ extern "C" {
 #include "lib-common-externs.h"
 
 /*@unused@*/
-//static char cvsid[] = "$Id: newt-specific.c 586 2006-05-29 09:36:33Z bcornec $";
+//static char cvsid[] = "$Id: newt-specific.c 877 2006-10-05 21:47:22Z andree $";
 
 	extern pid_t g_mastermind_pid;
 	extern char *g_tmpfs_mountpt;
 	extern char *g_boot_mountpt;
 	extern char *g_mondo_home;
+	extern char *ps_options;
 
 	extern void set_signals(int);
 
@@ -339,7 +340,17 @@ extern "C" {
 		g_current_progress = -999;
 	}
 
+/**
+ * Kill any process containing the string @p str surrounded by spaces in its commandline.
+ */
+void kill_anything_like_this(char *str) {
 
+char *tmp = NULL;
+
+asprintf(&tmp,"kill `ps %s | grep \" %s \" | awk '{print $1;}' | grep -vx \"\\?\"`", ps_options, str);
+run_program_and_log_output(tmp, TRUE);
+paranoid_free(tmp);
+}
 
 
 /**
@@ -915,7 +926,8 @@ extern "C" {
 			}
 		}
 
-		strcpy(prompt, p);
+		strncpy(prompt, p, MAX_NEWT_COMMENT_LEN - 1);
+		prompt[MAX_NEWT_COMMENT_LEN] = '\0';
 		text = newtTextboxReflowed(1, 1, prompt, 40, 5, 5, 0);
 		b_1 =
 			newtButton(20 -
@@ -1604,9 +1616,11 @@ extern "C" {
 		void *keylist[ARBITRARY_MAXIMUM];
 
 		/*@ int ************************************************************* */
-		int i = 0;
 		int currline = 0;
 		int finished = FALSE;
+
+		/*@ long ************************************************************ */
+		long i = 0;
 		long lng = 0;
 
 		/*@ buffers ********************************************************* */
@@ -1656,7 +1670,7 @@ extern "C" {
 								   keylist[i]);
 		}
 		sprintf(differ_sz,
-				"  %d files differ. Hit 'Select' to pick a file. Hit 'Close' to quit the list.",
+				"  %ld files differ. Hit 'Select' to pick a file. Hit 'Close' to quit the list.",
 				i);
 		newtPushHelpLine(differ_sz);
 		bClose = newtCompactButton(10, 15, " Close  ");
