@@ -7,7 +7,7 @@ copyright : (C) 2002 Mondo  Hugo Rabson
 email     : Hugo Rabson <hugorabson@msn.com>
 edited by : by Stan Benoit 4/2002
 email     : troff@nakedsoul.org
-cvsid     : $Id: mondo-prep.c 576 2006-05-25 12:00:37Z bcornec $
+cvsid     : $Id: mondo-prep.c 746 2006-08-05 10:00:51Z bruno $
 ***************************************************************************/
 
 /***************************************************************************
@@ -180,7 +180,7 @@ cvsid     : $Id: mondo-prep.c 576 2006-05-25 12:00:37Z bcornec $
 #define ARCHIVES_PATH MNT_CDROM"/archives"
 #define MONDO_WAS_HERE "MONDOWOZEREMONDOWOZEREMONDOWOZEREhahahaMOJOJOJO"
 
-//static char cvsid[] = "$Id: mondo-prep.c 576 2006-05-25 12:00:37Z bcornec $";
+//static char cvsid[] = "$Id: mondo-prep.c 746 2006-08-05 10:00:51Z bruno $";
 
 extern char *g_mountlist_fname;
 extern long g_current_progress, g_maximum_progress;
@@ -413,8 +413,7 @@ int do_my_funky_lvm_stuff(bool just_erase_existing_volumes,
 	sleep(1);
 	sprintf(command,
 			"for i in `%s | grep -i lvm | cut -d'\"' -f2` ; do %s -a n $i ; %s $i; echo \"Shutting down vg $i\" >> "
-			MONDO_LOGFILE "; done; %s -a n", vgscan_sz, vgchange_sz,
-			vgremove_sz, vgremove_sz);
+			MONDO_LOGFILE "; done", vgscan_sz, vgchange_sz, vgremove_sz);
 	run_program_and_log_output(command, 5);
 	if (just_erase_existing_volumes) {
 		paranoid_fclose(fin);
@@ -1818,7 +1817,7 @@ int partition_drive(struct mountlist_itself *mountlist, char *drivename)
 		system(tmp);
 		log_msg(0,
 				"------------------- end of fdisk.log... word! ------------------");
-		sprintf(tmp, "tail -n6 %s | fgrep \"16: \"", FDISK_LOG);
+		sprintf(tmp, "tail -n6 %s | grep -F \"16: \"", FDISK_LOG);
 		if (!run_program_and_log_output(tmp, 5)) {
 			g_partition_table_locked_up++;
 			log_to_screen
@@ -2323,7 +2322,6 @@ int start_raid_device(char *raid_device)
 	sprintf(program, "vinum start -f %s", raid_device);
 #else
 	sprintf(program, "raidstart %s", raid_device);
-//      sprintf (program, "raidstart " RAID_DEVICE_STUB "*");
 #endif
 	log_msg(1, "program = %s", program);
 	res = run_program_and_log_output(program, 1);
