@@ -4,7 +4,7 @@
     begin                : Fri Apr 25 2003
     copyright            : (C) 2000 by Hugo Rabson
     email                : Hugo Rabson <hugorabson@msn.com>
-    cvsid                : $Id: mondo-rstr-compare.c 128 2005-11-19 01:27:41Z bcornec $
+    cvsid                : $Id: mondo-rstr-compare.c 541 2006-05-13 18:47:23Z bcornec $
 ***************************************************************************/
 
 /***************************************************************************
@@ -64,7 +64,7 @@
 #include "mondo-restore-EXT.h"
 #include "mondo-rstr-tools-EXT.h"
 
-//static char cvsid[] = "$Id: mondo-rstr-compare.c 128 2005-11-19 01:27:41Z bcornec $";
+//static char cvsid[] = "$Id: mondo-rstr-compare.c 541 2006-05-13 18:47:23Z bcornec $";
 
 void popup_changelist_from_file(char *);
 
@@ -375,7 +375,7 @@ int compare_a_tarball(char *tarball_fname, int current_tarball_number)
 	}
 	if (length_of_file(logfile) > 5) {
 		sprintf(command,
-				"cat %s | sed s/': \\\"'/\\|/ | sed s/'\\\": '/\\|/ | cut -d'|' -f2 | sort | uniq | grep -vx \"dev/.*\" >> /tmp/changed.txt",
+				"sed s/': \\\"'/\\|/ %s | sed s/'\\\": '/\\|/ | cut -d'|' -f2 | sort -u | grep -vx \"dev/.*\" >> /tmp/changed.txt",
 				logfile);
 		system(command);
 		archiver_errors = count_lines_in_file(logfile);
