@@ -399,7 +399,7 @@ bool is_file_in_list(char *f, char *list_fname, char *preamble)
 			"Checking to see if f=%s, file=%s, is in the list of biggiefiles",
 			f, file);
 	log_msg(2, tmp);
-	sprintf(command, "cat %s | grep -x \"%s\"", list_fname, file);
+	sprintf(command, "grep -x \"%s\" %s", file, list_fname);
 	res = run_program_and_log_output(command, FALSE);
 	paranoid_free(command);
 	paranoid_free(file);
@@ -512,7 +512,7 @@ void kill_petris(void)
 	char *command;
 	malloc_string(command);
 	sprintf(command,
-			"kill `ps wax 2> /dev/null | grep petris 2> /dev/null | grep -v grep | cut -d' ' -f2` 2> /dev/null");
+			"kill `ps 2> /dev/null | grep petris 2> /dev/null | grep -v grep | cut -d' ' -f2` 2> /dev/null");
 	paranoid_system(command);
 	paranoid_free(command);
 }
@@ -557,7 +557,7 @@ int modify_rclocal_one_time(char *path)
 		return (1);
 	}
 	sprintf(newfile_fname, "%s/rc.local.mondorescue", path);
-	sprintf(tmp, "cat %s | grep mondorescue > /dev/null 2> /dev/null",
+	sprintf(tmp, "grep mondorescue %s > /dev/null 2> /dev/null",
 			rclocal_fname);
 	if (system(tmp)) {
 		sprintf(tmp, "echo \"[ -e %s ] && %s\n\" >> %s",
@@ -568,7 +568,7 @@ int modify_rclocal_one_time(char *path)
 	sprintf(tmp, "echo -en \"#!/bin/sh\
 \\n\
 \\n\
-cat %s | grep -v mondorescue > %s\\n\
+grep -v mondorescue %s > %s\\n\
 rm -f /var/lock/subsys/*xfs*\\n\
 rm -f /var/run/xfs.*\\n\
 killall xfs\\n\
@@ -719,7 +719,7 @@ int mount_cdrom(struct s_bkpinfo *bkpinfo)
 		return 0;
 	}
 
-	if (!run_program_and_log_output("mount | fgrep " MNT_CDROM, FALSE)) {
+	if (!run_program_and_log_output("mount | grep -F " MNT_CDROM, FALSE)) {
 		log_msg(2, "mount_cdrom() - CD already mounted. Fair enough.");
 		paranoid_free(mount_cmd);
 		return (0);
@@ -947,7 +947,7 @@ void protect_against_braindead_sysadmins()
 	run_program_and_log_output("chmod 777 " MNT_RESTORING "/dev/null",
 							   FALSE);
 	run_program_and_log_output("cd " MNT_RESTORING
-							   "; for i in `ls home/`; do echo \"Moving $i's spurious files to $i/.disabled\"; mkdir $i/.disabled ; mv -f $i/.DCOP* $i/.MCOP* $i/.*authority $i/.kde/tmp* $i/.kde/socket* $i/.disabled/ ; done",
+							   "; for i in `ls home/`; do echo \"Moving $i's spurious files to $i/.disabled\"; mkdir \"$i\"/.disabled ; mv -f \"$i\"/.DCOP* \"$i\"/.MCOP* \"$i\"/.*authority \"$i\"/.kde/tmp* \"$i\"/.kde/socket* \"$i\"/.disabled/ ; done",
 							   TRUE);
 	run_program_and_log_output("rm -f " MNT_RESTORING "/var/run/*.pid",
 							   TRUE);
@@ -972,12 +972,14 @@ void protect_against_braindead_sysadmins()
 int read_cfg_file_into_bkpinfo(char *cfgf, struct s_bkpinfo *bkpinfo)
 {
   /** add mallocs **/
-	char *value;
-	char *tmp;
-	char *command;
-	char *iso_mnt;
-	char *iso_path;
-	char *old_isodir;
+	char *value = NULL;
+	char *tmp = NULL;
+	char *envtmp1 = NULL;
+	char *envtmp2 = NULL;
+	char *command = NULL;
+	char *iso_mnt = NULL;
+	char *iso_path = NULL;
+	char *old_isodir = NULL;
 	char cfg_file[100];
 	t_bkptype media_specified_by_user;
 
@@ -1043,6 +1045,17 @@ int read_cfg_file_into_bkpinfo(char *cfgf, struct s_bkpinfo *bkpinfo)
 			} else {
 					strcpy(bkpinfo->prefix,STD_PREFIX);
 			}
+			/* We need to override prefix value in PXE mode as it's 
+			 * already done in start-nfs */
+			envtmp1 = getenv("imgname");
+			if (envtmp1 == NULL) {
+				fatal_error("no imgname variable in environment");
+			}
+			if (strstr(call_program_and_get_last_line_of_output
+			   ("cat /proc/cmdline"), "pxe")) {
+					strcpy(bkpinfo->prefix,envtmp1);
+			}
+
 		} else if (!strcmp(value, "tape")) {
 			bkpinfo->backup_media_type = tape;
 		} else if (!strcmp(value, "udev")) {
@@ -1150,6 +1163,21 @@ int read_cfg_file_into_bkpinfo(char *cfgf, struct s_bkpinfo *bkpinfo)
 			log_msg(2, "nfs_mount is %s", bkpinfo->nfs_mount);
 			log_msg(2, "nfs_remote_dir is %s", bkpinfo->nfs_remote_dir);
 		}
+		/* We need to override values in PXE mode as it's 
+		 * already done in start-nfs */
+		envtmp1 = getenv("nfsmount");
+		if (envtmp1 == NULL) {
+			fatal_error("no nfsmount variable in environment");
+		}
+		envtmp2 = getenv("dirimg");
+		if (envtmp2 == NULL) {
+			fatal_error("no dirimg variable in environment");
+		}
+		if (strstr(call_program_and_get_last_line_of_output
+		   ("cat /proc/cmdline"), "pxe")) {
+				strcpy(bkpinfo->nfs_mount,envtmp1);
+				strcpy(bkpinfo->nfs_remote_dir,envtmp2);
+			}
 	} else if (bkpinfo->backup_media_type == iso) {
 		/* Patch by Conor Daly 23-june-2004 
 		 * to correctly mount iso-dev and set a sensible
@@ -1201,7 +1229,7 @@ int read_cfg_file_into_bkpinfo(char *cfgf, struct s_bkpinfo *bkpinfo)
 							("Unable to mount isodir. Failed to mount CD-ROM as well.");
 					} else {
 						log_msg(1,
-								"You backed up to disk, then burned some CDs. Naughty monkey!");
+								"You backed up to disk, then burned some CDs.");
 					}
 				}
 			}
@@ -1370,7 +1398,7 @@ s_node *process_filelist_and_biggielist(struct s_bkpinfo *bkpinfo)
 			sprintf(command, "> %s", g_biggielist_txt);
 			paranoid_system(command);
 		}
-		sprintf(command, "cat %s | grep  -x \"/dev/.*\" > %s",
+		sprintf(command, "grep  -x \"/dev/.*\" %s > %s",
 				g_biggielist_txt, g_filelist_imagedevs);
 		paranoid_system(command);
 		exit(0);
@@ -2077,77 +2105,6 @@ void streamline_changes_file(char *output_file, char *input_file)
  **************************************************************************/
 
 
-
-
-
-
-/**
- * Prompt the user to support the Mondo project.
- */
-void success_message(void)
-{
-	int i;
-	/* malloc and ptr */
-	char *tmp;
-
-	malloc_string(tmp);
-	if (strstr
-		(call_program_and_get_last_line_of_output("cat /proc/cmdline"),
-		 "restore") == NULL) {
-		if (ask_me_yes_or_no
-			("Have you contributed to the Mondo project financially or in some other way, yet?"))
-		{
-			log_to_screen
-				("Thank you for supporting Mondo. It goes from strength to strength,");
-			log_to_screen("thanks to the support of users like you.");
-		} else {
-			if (ask_me_yes_or_no
-				("Are you or your company willing to consider contributing to Mondo in some way?"))
-			{
-				popup_and_OK
-					("To support the project which has just performed a valuable service for you, please visit http://www.mondorescue.com; click on 'Download' and then 'PayPal'.");
-			} else {
-				log_to_screen
-					("Free Software, like freedom itself, must be supported or it will be lost.");
-				log_msg(1,
-						"To your credit, you were honest: you said no, you wouldn't be contributing");
-				log_msg(1,
-						"to this project, ever. However, that makes you a freeloader. I bet you're");
-				log_msg(1,
-						"the sort of person who likes to sneak into movie theatres...");
-				popup_and_OK
-					("If you ever change your mind, you may support this product by going to http://www.mondrescue.com and clicking on 'Download', followed by 'PalPal'.");
-			}
-		}
-	}
-#ifdef FREELOADER
-	i = (int) (random()) % 32;
-#else
-	i = 1;
-#endif
-
-	if (i != 25) {
-		strcpy(tmp,
-			   "Mondo has restored your system. Please remove the backup media and reboot.");
-	} else {
-		strcpy(tmp,
-			   "M0nd0 h45 r3570r3d j00r 5y573m. P13453 r3m0v3 7h3 b4ckup m3d14 4nd r3b007.");
-	}
-	if (strstr
-		(call_program_and_get_last_line_of_output("cat /proc/cmdline"),
-		 "restore") == NULL) {
-		popup_and_OK(tmp);
-	}
-	log_to_screen(tmp);
-	paranoid_free(tmp);
-}
-
-/**************************************************************************
- *END_SUCCESS_MESSAGE                                                     *
- **************************************************************************/
-
-
-
 /**
  * Exit due to a signal (normal cleanup).
  * @param sig The signal we're exiting due to.
@@ -2249,7 +2206,7 @@ int unmount_all_devices(struct mountlist_itself
 		 i < 10
 		 &&
 		 run_program_and_log_output
-		 ("ps wax | grep buffer | grep -v \"grep buffer\"", TRUE) == 0;
+		 ("ps | grep buffer | grep -v \"grep buffer\"", TRUE) == 0;
 		 i++) {
 		sleep(1);
 		log_msg(2, "Waiting for buffer() to finish");
@@ -2427,7 +2384,7 @@ int get_cfg_file_from_archive(struct s_bkpinfo *bkpinfo)
 
 	sprintf(command, "mount /dev/fd0u1722 %s", mountpt);
 	sprintf(tmp,
-			"(sleep 15; kill `ps ax | grep \"%s\" | cut -d' ' -f1` 2> /dev/null) &",
+			"(sleep 15; kill `ps | grep \"%s\" | cut -d' ' -f1` 2> /dev/null) &",
 			command);
 	log_msg(1, "tmp = '%s'", tmp);
 	system(tmp);
@@ -2655,34 +2612,39 @@ int get_cfg_file_from_archive(struct s_bkpinfo *bkpinfo)
 void wait_until_software_raids_are_prepped(char *mdstat_file,
 										   int wait_for_percentage)
 {
-	struct s_mdstat *mdstat;
+	struct raidlist_itself *raidlist;
 	int unfinished_mdstat_devices = 9999, i;
 	char *screen_message;
 
 	malloc_string(screen_message);
-	mdstat = malloc(sizeof(struct s_mdstat));
+	raidlist = malloc(sizeof(struct raidlist_itself));
 
 	assert(wait_for_percentage <= 100);
-	iamhere("Help, my boat is sync'ing. (Get it? Urp! Urp!)");
+	iamhere("wait_until_software_raids_are_prepped");
 	while (unfinished_mdstat_devices > 0) {
-		if (read_mdstat(mdstat, mdstat_file)) {
-			log_to_screen("Sorry, cannot read %s", mdstat_file);
+	        // FIXME: Prefix '/dev/' should really be dynamic!
+		if (parse_mdstat(raidlist, "/dev/")) {
+			log_to_screen("Sorry, cannot read %s", MDSTAT_FILE);
+			log_msg(1,"Sorry, cannot read %s", MDSTAT_FILE);
 			return;
 		}
-		for (unfinished_mdstat_devices = i = 0; i < mdstat->entries; i++) {
-			if (mdstat->el[i].progress < wait_for_percentage) {
+		for (unfinished_mdstat_devices = i = 0; i <= raidlist->entries; i++) {
+			if (raidlist->el[i].progress < wait_for_percentage) {
 				unfinished_mdstat_devices++;
-				sprintf(screen_message, "Sync'ing /dev/md%d",
-						mdstat->el[i].md);
-				open_evalcall_form(screen_message);
-				if (mdstat->el[i].progress == -1)	// delayed while another partition inits
+				if (raidlist->el[i].progress == -1)	// delayed while another partition inits
 				{
 					continue;
 				}
-				while (mdstat->el[i].progress < wait_for_percentage) {
-					update_evalcall_form(mdstat->el[i].progress);
+				log_msg(1,"Sync'ing %s (i=%d)", raidlist->el[i].raid_device, i);
+				sprintf(screen_message, "Sync'ing %s",
+						raidlist->el[i].raid_device);
+				open_evalcall_form(screen_message);
+				while (raidlist->el[i].progress < wait_for_percentage) {
+					log_msg(1,"Percentage sync'ed: %d", raidlist->el[i].progress);
+					update_evalcall_form(raidlist->el[i].progress);
 					sleep(2);
-					if (read_mdstat(mdstat, mdstat_file)) {
+					// FIXME: Prefix '/dev/' should really be dynamic!
+					if (parse_mdstat(raidlist, "/dev/")) {
 						break;
 					}
 				}
@@ -2691,5 +2653,5 @@ void wait_until_software_raids_are_prepped(char *mdstat_file,
 		}
 	}
 	paranoid_free(screen_message);
-	paranoid_free(mdstat);
+	paranoid_free(raidlist);
 }
